from __future__ import annotations

import typing as t

import pandas as pd

from .exceptions import TableHandlerError


class TableHandler:
    """Table data handler for the application a wrapper on top of dataframes"""

    def __init__(self, df: pd.DataFrame, schema: t.List, label: str) -> None:
        """
        Initializer of the TableHandler class
        :param df: A pandas dataframe
        :param schema: A Column Name list of the DataFrame to use
        :param label: A label/tag which will be used as excel sheet name when creating a excel file
        """
        self.df = df
        self.schema = schema
        self.label = label

    @classmethod
    def from_records(
        cls,
        data: t.List[t.Dict],
        col_headers: t.Optional[t.OrderedDict[str, str]],
        label: str,
    ) -> TableHandler:
        """
        Create A TableHandler object from list of records
        :param data: List of records, similar structure as pd.DataFrame().to_dict(orient="records")
        :param col_headers: Column Rename Map, if Not provided default DataFrame column will be used
        :param label: A label/tag which will be used when creating sheet name in excel file
        :return: Object of type TableHandler
        """
        try:
            df = pd.DataFrame(data)
            if col_headers:
                schema = list(col_headers.values())
                df = df.rename(columns=dict(col_headers))
                df = df[schema]

                return cls(df=df, schema=schema, label=label)
            return cls(df=df, schema=list(df.columns), label=label)
        except Exception as e:
            raise TableHandlerError(e)

    @classmethod
    def from_list(
        cls, data: t.List, col_headers: t.Optional[t.List[str]], label: str
    ) -> TableHandler:
        """
        Create A TableHandler object from list of list e.g data = [[...], [...]] or data = [(...), (...)]
        :param data: list of a list of rows to create DataFrame
        :param col_headers: column name to use for the DataFrame, if not provided default will be used
        :param label: A label/tag which will be used when creating sheet name in excel file
        """
        try:
            df = pd.DataFrame(data)
            if col_headers:
                if len(col_headers) != len(df.columns):
                    raise TableHandlerError(
                        "Length of col_headers does not match with the length of DataFrame columns"
                    )
                df.columns = col_headers
            schema = list(df.columns)
            return cls(df=df, schema=schema, label=label)

        except Exception as e:
            raise TableHandlerError(e)

    @classmethod
    def from_file_like(cls, content: t.Union[t.TextIO, t.BinaryIO]) -> TableHandler:
        """
        Create itself from a file like object from streamlit
        :param content: File like object from frontend
        """
        try:
            all_sheets = pd.ExcelFile(content)
            sheet_name = all_sheets.sheet_names[0]
            df = pd.read_excel(content, sheet_name=sheet_name)
            schema = list(df.columns)
            schema = [str(item) for item in schema]
            label = sheet_name
            return cls(df, schema, label)
        except TableHandlerError as e:
            raise TableHandlerError(e)
        except Exception as e:
            raise TableHandlerError(e)

    def get_data_as_dataframe(self) -> pd.DataFrame:
        """Get the DataFrame store in the class"""
        return self.df

    def get_label(self) -> str:
        """Getter for the label field"""
        return self.label

    def get_schema(self) -> t.List:
        """Getter for the schema field"""
        return self.schema

    def get_data_as_records(self) -> t.List[t.Dict]:
        """Get the records stored in the dataframe as List of Dict"""
        return self.df.to_dict(orient="records")

    def __str__(self) -> str:
        """String Representation of the class"""
        return f"{self.__class__.__name__}(df=({self.df.shape}), {self.schema}, {self.label})"

    def __repr__(self) -> str:
        """Dev string representation of the class"""
        return f"{self.__class__.__name__}({self.df}, {self.schema}, {self.label})"

    def __len__(self) -> int:
        """Len representation provides the len of the DataFrame attached"""
        return len(self.df)



from __future__ import annotations

import base64
import io
import os
import typing as t
from io import BytesIO

from docx import Document as ReadDocFunc
from docx.document import Document

from .exceptions import DocxHandlerError
from .handler import Handler
from .html import create_download_link


class DocxHandler(Handler):
    def __init__(self, document: Document, name: str) -> None:
        """
        Handle all the conversion to and from document file
        :param document: A docx file like object
        """
        self.document = document
        self.name = name

    def get_stat(self) -> t.Dict:
        total_tables = len([table for table in self.document.tables])
        total_paragraphs = self.__len__()
        return {"tables": total_tables, "paragraphs": total_paragraphs}

    def to_byte(self) -> bytes:
        """
        Convert a Python Docx file into byte
        :param doc: Python Document type from Docx
        :return: Bytes of Document
        """
        byte_stream = BytesIO()
        self.document.save(byte_stream)
        byte_stream.seek(0)
        return byte_stream.read()

    def to_base64_str(self) -> str:
        """
        Convert bytes data to base64
        :param byte_data: data as byte
        :return: base 64 encoded string of the byte data
        """
        byte_data = self.to_byte()
        b64_encoded_str = base64.b64encode(byte_data).decode("utf-8")
        return b64_encoded_str

    @classmethod
    def from_base64(cls, b64_str: str, filename: t.Optional[str] = None) -> DocxHandler:
        """
        Create a instance of DocxHandler from base64 encoded string
        :return: a DocxHandler object
        """
        bytes_stream = base64.b64decode(b64_str.encode("utf-8"))
        document = ReadDocFunc(io.BytesIO(bytes_stream))
        if filename:
            return cls(document=document, name=filename)
        return cls(document=document, name="document")

    def write_to_local(self, path: str, filename: str) -> None:
        full_path = os.path.join(path, filename)
        self.document.save(full_path)

    @classmethod
    def from_file_like(cls, content: t.Union[t.TextIO, t.BinaryIO]) -> DocxHandler:
        """
        Create itself from a file like object from framework like Streamlit/Dash
        """

        try:
            document = ReadDocFunc(content)
            return cls(document=document, name=content.name)

        except DocxHandlerError:
            raise DocxHandlerError
        except Exception as e:
            raise e

    def create_download_link(self, filename: str) -> str:
        """
        Create a html annotated tag to show in the front end
        :param filename: Name of the File when downloaded
        :return: Html anchor tag annotated String
        """

        content = self.to_base64_str()
        return create_download_link(
            base64_str=content, filename=filename, filetype="docx"
        )

    def __str__(self) -> str:
        """String representation of the class"""
        return f'{self.__class__.__name__}(document="...", name={self.name})'

    def __repr__(self):
        raise NotImplementedError

    def __len__(self) -> int:
        """Determine if there is any table in the docx or not"""
        return len(
            [para for para in self.document.paragraphs if len(para.text.strip()) > 0]
        )



from abc import ABC, abstractmethod


class Handler(ABC):
    """A Abstract handler interface"""

    @abstractmethod
    def to_base64_str(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def __str__(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def __repr__(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def __len__(self) -> int:
        raise NotImplementedError



from .exceptions import FileTypeError


def create_download_link(base64_str: str, filename: str, filetype: str) -> str:
    accepted_type = ["docx", "xlsx"]
    if filetype not in accepted_type:
        raise FileTypeError(filetype=filetype)
    return f'<a href="data:application/octet-stream;base64,{base64_str}" download="{filename}">Click To Download</a>'



from setuptools import find_packages, setup

from setup_utils import SetupConfig

setup(
    name="xfilios",
    version="0.0.2",
    author="Data Psycho",
    author_email="mr.data.psycho@gmail.com",
    url="https://github.com/DataPsycho/xfilios",
    description="Docx and Excel File handler for Streamlit or Dash",
    long_description=SetupConfig.read_metadata_from_readme(),
    long_description_content_type="text/markdown",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    install_requires=SetupConfig.get_install_requirements(),
    license="MIT",
    license_files=("LICENSE",),
    keywords="FileIO, Docx, Excel, Streamlit, Dash",
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Development Status :: 4 - Beta",
        "Environment :: MacOS X",
    ],
    zip_safe=False,
    include_package_data=True,
)



from __future__ import annotations

import base64
import io
import typing as t

import pandas as pd

from .handler import Handler
from .html import create_download_link
from .table import TableHandler


class ExcelHandler(Handler):
    """Excel handler to create downloadable excel file using Excel Writer"""

    def __init__(self, tables: t.List[TableHandler]):
        """
        Excel Handler objects init
        :param tables: list of TableHandler instances to convert into Excel downloadable link
        """
        self.tables = tables

    @classmethod
    def from_table_handlers(cls, *args: TableHandler) -> ExcelHandler:
        """
        Create ExcelHandler from list of table Handlers
        :param args: List of TableHandler Object
        :return: Instance of Type ExcelHandler
        """
        handlers = [handler for handler in args]
        return cls(tables=handlers)

    def to_base64_str(self) -> str:
        """
        Convert table handlers into base64 encoded downloadable string
        """
        with io.BytesIO() as output:
            writer = pd.ExcelWriter(output)
            for table in self.tables:
                df = table.get_data_as_dataframe()
                label = table.get_label()
                df.to_excel(writer, sheet_name=label, index=False)
            writer.save()
            output.seek(0)
            decoded_data = base64.b64encode(output.read()).decode("utf-8")
            return decoded_data

    def create_download_link(self, filename: str) -> str:
        """
        Create download links from base 64 encoded string for the frontend
        :param filename: filename to be used when creating downloadable link
        """
        content = self.to_base64_str()
        return create_download_link(
            base64_str=content, filename=filename, filetype="xlsx"
        )

    def __str__(self) -> str:
        """String representation of the class"""
        return f"{self.__class__.__name__}([TableHandler(...), ...])"

    def __repr__(self) -> str:
        """Dev string representation of the class"""
        return f"{self.__class__.__name__}({self.tables})"

    def __len__(self) -> int:
        """Len implementation provides total number of TableHandler"""
        return len(self.tables)



import typing as t


class FileTypeError(Exception):
    filetypes = ["docx", "xlsx"]

    def __init__(self, filetype: str) -> None:
        self.filetype = filetype
        self.message = self.create_message()
        super().__init__(self.message)

    def create_message(self) -> str:
        text1 = f"{self.filetype} is not a excepted file type."
        text2 = "Only excepted file types are {}".format(", ".join(self.filetypes))
        return f"{text1} {text2}"


class TableHandlerError(Exception):
    def __init__(self, message: t.Optional[t.Union[Exception, str]] = None) -> None:
        self.message = (
            "Could not read the content from excel file, either no excel sheet "
            "provided, or it is malformed."
        )
        if message:
            self.message = str(message)
        super().__init__(self.message)


class DocxHandlerError(Exception):
    def __init__(self, message: t.Optional[t.Union[Exception, str]] = None) -> None:
        self.message = "Could not read the content from docx file, either no content in the file or it is malformed."
        if message:
            self.message = str(message)
        super().__init__(self.message)
